#include <stdio.h>
#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>
#include <time.h>
#include <cmocka.h>

#include "mtime_calendar.h"
#include "mtime_datetime.h"
#include "mtime_date.h"
#include "mtime_time.h"
#include "mtime_timedelta.h"
#include "mtime_eventHandling.h"
#include "mtime_eventList.h"

#include "mtime_julianDay.h"

/**
 * check the string representation of mtime variables of type
 * datetime and timedelta
 */
static void test_mtime_string_representation() {

  struct _timedelta *td;
  struct _datetime *start;
  char tds[MAX_TIMEDELTA_STR_LEN];

  /* check datetimes with milliseconds */
  start = newDateTime("1888-12-12T22:58:44.1");
  assert_true(1.0 == start->time.ms);
  start = newDateTime("1888-12-12T22:58:44.01");
  assert_true(1.0 == start->time.ms);
  start = newDateTime("1888-12-12T22:58:44.001");
  assert_true(1.0 == start->time.ms);

  /*
  assert_string_equal("1888-12-12T22:58:44.01",datetimeToString(start, starts));
  assert_string_not_equal("1888-12-12T22:58:44.001",datetimeToString(start, starts));
  */

  /* string representation od time delta adds '0' */
  td = newTimeDelta("P1D");
  assert_string_equal("P01D",timedeltaToString(td,tds));
  assert_string_not_equal("P1D",timedeltaToString(td,tds));

  td = newTimeDelta("PT1M");
  assert_string_equal("PT01M",timedeltaToString(td,tds));
  assert_string_not_equal("PT1M",timedeltaToString(td,tds));

  td = newTimeDelta("P1Y6M");
  assert_string_equal("P1Y06M",timedeltaToString(td,tds));

}

/**
 * test the addition of a month uppon the and of january
 */
static void test_mtime_add_months () {

  int i;
  struct _timedelta *td;
  struct _datetime *ref;
  char curs[MAX_DATETIME_STR_LEN];

  ref    = newDateTime("1999-12-31T23:50:00");
  td     = newTimeDelta("P1M");

  struct _datetime* cur = constructAndCopyDateTime(ref);

  for (i = 0; i < 14; i++)
  {
    /* compute month shift */
    addTimeDeltaToDateTime(cur,td,cur);

#ifdef EBUG
    printf("             Current date is: %s\n",datetimeToString(cur, curs));
#endif
    if (i == 0 ) assert_string_equal("2000-01-31T23:50:00.000",datetimeToString(cur,curs));
    if (i == 1 ) assert_string_equal("2000-03-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 2 ) assert_string_equal("2000-04-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 3 ) assert_string_equal("2000-05-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 4 ) assert_string_equal("2000-06-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 5 ) assert_string_equal("2000-07-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 6 ) assert_string_equal("2000-08-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 7 ) assert_string_equal("2000-09-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 8 ) assert_string_equal("2000-10-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 9 ) assert_string_equal("2000-11-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 10) assert_string_equal("2000-12-02T23:50:00.000",datetimeToString(cur,curs));
    if (i == 11) assert_string_equal("2001-01-02T23:50:00.000",datetimeToString(cur,curs));
  }
}

/**
 * test the addition of a month uppon the and of january
 *
 * the goal is to mimic a list dates, which are at the end of a month
 */
static void test_mtime_add_months_atEnd () {

  int i;
  struct _timedelta *td, *offset;
  struct _datetime *start, *ref;
  char curs[MAX_DATETIME_STR_LEN];

  start  = newDateTime("2001-01-01T00:00:00");
  ref    = newDateTime("1999-12-31T23:50:00");
  td     = newTimeDelta("P1M");
  offset = newTimeDelta("-PT10M");

  ref    = start;

  struct _datetime* cur = constructAndCopyDateTime(ref);
  /*
  printf("Current date is: %s\n",datetimeToString(cur, curs));
  */

  for (i = 0; i < 14; i++)
  {
    /* compute month shift */
    addTimeDeltaToDateTime(cur,td,cur);
    /* subtract 10 minutes */
    struct _datetime* cur_backshift = constructAndCopyDateTime(cur);
    addTimeDeltaToDateTime(cur_backshift,offset,cur_backshift);

#ifdef EBUG
    printf("             Current date is: %s\n",datetimeToString(cur_backshift, curs));
#endif

    switch (i%12) {
      case 0: case 2: case 4: case 6: case 7: case 9: case 11:
        assert_int_equal(31, cur_backshift->date.day);
        break;
      case 1:
        assert_int_equal(28, cur_backshift->date.day);
        break;
      case 3: case 5: case 8: case 10:
        assert_int_equal(30, cur_backshift->date.day);
        break;
      default:
        printf(" error - this should never happen\n");
        break;
    }
  }
}

/**
 * test events created with offset to trigger 10 minutes before end of months
 */
static void test_event_at_endOfMonths() {
  /* some lines stolen from model_integrate.c */
  char current_time[MAX_DATETIME_STR_LEN];  // Again, we use *STR_LEN.
  char current_step[MAX_TIMEDELTA_STR_LEN]; // and here too.
  struct _timedelta *timestep  = newTimeDelta("PT10M");
  struct _datetime *dummy;
  struct _datetime *start_date = newDateTime("2013-01-01T00:00:00.000");
  struct _datetime *stop_date  = newDateTime("2014-03-03T14:00:00.000");
  struct _datetime *model_time = newDateTime("2013-01-01T00:00:00.000");

  struct _event* out = newEvent("outputEvent",
                                "2013-01-01T00:00:00.000",
                                "2013-01-01T00:00:00.000",
                                "2019-01-01T00:00:00.000",
                                "P1M",
                                 NULL);
  struct _event* outAtEnd = newEvent("outputEventAtEnd",
                                     "2012-12-31T23:50:00.000",
                                     "2013-01-01T00:00:00.000",
                                     "2019-01-01T00:00:00.000",
                                     "P1M",
                                     NULL);

#ifdef EBUG
  printf("Model time step: %s\n", timedeltaToString(timestep, current_step));
  printf("Model start time: %s\n\n", datetimeToString(model_time, current_time));
#endif

  do
    {
      /* check if outputEvent is active */
      dummy = addTimeDeltaToDateTime(model_time,newTimeDelta("PT10"),dummy);
      bool activeOut      = isCurrentEventActive(out,model_time, NULL, NULL);
      bool activeOutShift = isCurrentEventActive(out,dummy , NULL, NULL);
      bool activeOutAtEnd = isCurrentEventActive(outAtEnd,model_time, NULL, NULL);
#ifdef EBUG
      if (activeOut || activeOutAtEnd || activeOutShift) printf("Model time: %s  ", datetimeToString(model_time, current_time));
      if (activeOut)                   printf("outputEvent active! ");
      if (activeOutAtEnd)              printf("outputEventAtEnd active!");
      if (activeOutShift)              printf("outputShifted    active!");
      if (activeOut || activeOutAtEnd ) printf("\n");
#endif

      addTimeDeltaToDateTime(model_time, timestep, model_time); /* Increment time by timestep. */


    } while (compareDatetime(model_time, stop_date) <= 0);
}

/**
 * check adding negative time periods
 */
static void test_add_delta(){
  struct _datetime *dummy;
  struct _datetime *init = newDateTime("1111-11-11T00:00:00.000");
  struct _timedelta *delta = newTimeDelta("PT10M");
  struct _timedelta *minusdelta = newTimeDelta("-PT10M");
  char init_s[MAX_DATETIME_STR_LEN];

  assert_string_equal("1111-11-11T00:10:00.000",datetimeToString(addTimeDeltaToDateTime(init,delta,init),init_s));
  assert_string_equal("1111-11-11T00:20:00.000",datetimeToString(addTimeDeltaToDateTime(init,delta,init),init_s));
  assert_string_equal("1111-11-11T00:10:00.000",datetimeToString(addTimeDeltaToDateTime(init,minusdelta,init),init_s));
  assert_string_equal("1111-11-11T00:00:00.000",datetimeToString(addTimeDeltaToDateTime(init,minusdelta,init),init_s));
}

int main(void) {
  initCalendar(PROLEPTIC_GREGORIAN);

  const struct CMUnitTest tests[] = {
    cmocka_unit_test(test_mtime_string_representation),
    cmocka_unit_test(test_mtime_add_months),
    cmocka_unit_test(test_mtime_add_months_atEnd),
    cmocka_unit_test(test_event_at_endOfMonths),
    cmocka_unit_test(test_add_delta),
  };
  return cmocka_run_group_tests(tests, NULL, NULL);
}
